/*
 * libkarma/pathedit.c
 *
 * Pathname editing
 *
 * Copyright (c) Keith Bennett <keith@mcs.st-and.ac.uk> 2006
 *
 * You may distribute and modify this program under the terms of 
 * the GNU GPL, version 2 or later.
 */

#include <string.h>
#include "pathedit.h"

void unescape(char *str, char *escapefrom, char *escapeto)
{
    char *sptr, *ptr, *frm, *to;

    ptr = sptr = str;
    /* un-hidden files get re-hidden */
    if (*sptr == '_' && *(sptr+1) == '.') {
        *ptr++ = '.';
        sptr += 2;
    }
    for (; *sptr != '\0'; sptr++) {
        if (*sptr == TR_ESC) {
            sptr++;
            /* deal with special characters */
            switch (*sptr) {
                case TR_ESC: *ptr++ = *sptr; continue;
                case 'a': *ptr++ = '\a'; continue;
                case 'b': *ptr++ = '\b'; continue;
                case 'f': *ptr++ = '\f'; continue;
                case 'n': *ptr++ = '\n'; continue;
                case 'r': *ptr++ = '\r'; continue;
                case 't': *ptr++ = '\t'; continue;
                case 'v': *ptr++ = '\v'; continue;
            }
            *ptr++ = *sptr;
        } else {
            *ptr = *sptr;
            for (frm = escapefrom, to = escapeto; *frm != '\0'; frm++, to++) {
                if (*sptr == *to) {
                    *ptr = *frm;
                    break;
                }
            }
            ptr++;
        }
        
    }
    *ptr = '\0';
}

void escape(char *str, char *escapefrom, char *escapeto)
{
    int num = 0, edit = 0;
    char *ptr, *nptr, *frm, *to;

    /* hidden files get un-hidden */
    if (*str == '.') {
        edit = 1;
        num++;
    }

    /* count the number of characters to be escaped */
    for (ptr = str; *ptr != '\0'; ptr++) {
        /* deal with special characters */
        switch (*ptr) {
            case  TR_ESC:
            case '\a':
            case '\b':
            case '\f':
            case '\n':
            case '\r':
            case '\t':
            case '\v':
                edit = 1;
                num++;
                continue;
        }
        for (to = escapeto, frm = escapefrom; *to != '\0'; to++, frm++) {
            if (*ptr == *to) {
                edit = 1;
                num++;
                break;
            } else if (*ptr == *frm) {
                edit = 1;
                break;
            }
        }
    }

    if (edit == 0)
        return;

    /* overwrite the string, starting from the end and working backwards */
    nptr = str+strlen(str)+num+1;
    *nptr-- = '\0';
    for (ptr = str+strlen(str); ptr >= str; ptr--) {
        /* deal with special characters */
        switch (*ptr) {
            case  TR_ESC: *nptr-- = TR_ESC; *nptr-- = TR_ESC; continue;
            case '\a': *nptr-- = 'a'; *nptr-- = TR_ESC; continue;
            case '\b': *nptr-- = 'b'; *nptr-- = TR_ESC; continue;
            case '\f': *nptr-- = 'f'; *nptr-- = TR_ESC; continue;
            case '\n': *nptr-- = 'n'; *nptr-- = TR_ESC; continue;
            case '\r': *nptr-- = 'r'; *nptr-- = TR_ESC; continue;
            case '\t': *nptr-- = 't'; *nptr-- = TR_ESC; continue;
            case '\v': *nptr-- = 'v'; *nptr-- = TR_ESC; continue;
        }
        for (frm = escapefrom, to = escapeto; *frm != '\0'; frm++, to++) {
            if (*ptr == *frm) {
                *nptr-- = *to;
                goto next;
            } else if (*ptr == *to) {
                *nptr-- = *to;
                *nptr-- = TR_ESC;
                goto next;
            }
        }
        *nptr-- = *ptr;
next:
        continue;
    }

    /* hidden files get un-hidden */
    if (*str == '.')
        *str = '_';

    return;
}
